@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -37,11 +37,13 @@ DataManager::DataManager() :
     m_foldersState( 127 ),
     m_issuesState( 8191 )
 {
+    loadWatchCache();
 }
 
 DataManager::~DataManager()
 {
     saveFolderCache();
+    saveWatchCache();
 }
 
 void DataManager::addObserver( QObject* observer )
@@ -170,6 +172,43 @@ void DataManager::flushIssueCache()
     }
 }
 
+void DataManager::setFolderWatchState( int folderId, WatchState state )
+{
+    FolderState* fstate = folderState( folderId );
+    fstate->setWatchState( state );
+}
+
+WatchState DataManager::folderWatchState( int folderId )
+{
+    FolderState* fstate = folderState( folderId );
+    return fstate->watchState();
+}
+
+void DataManager::updateIssueWatchStamp( int issueId )
+{
+    IssueRow* issue = m_issues.find( issueId );
+    if ( issue != NULL ) {
+        IssueState* istate = issueState( issueId );
+        istate->setWatchStamp( issue->stamp() );
+    }
+}
+
+WatchState DataManager::issueWatchState( int issueId, bool ignoreFolder )
+{
+    const IssueRow* issue = m_issues.find( issueId );
+    if ( issue != NULL ) {
+        if ( ignoreFolder || folderWatchState( issue->folderId() ) != NotWatched ) {
+            IssueState* istate = issueState( issueId );
+            if ( istate->watchStamp() == 0 )
+                return WatchNew;
+            if ( istate->watchStamp() < issue->stamp() )
+                return WatchUpdated;
+            return WatchNormal;
+        }
+    }
+    return NotWatched;
+}
+
 Command* DataManager::updateUsers()
 {
     Command* command = new Command();
@@ -642,4 +681,78 @@ void DataManager::writeFolderCache( int folderId )
     stream << (qint32)0;
 }
 
-#include "datamanager.moc"
+void DataManager::loadWatchCache()
+{
+    QString path = connectionManager->locateCacheFile( "watch.cache" );
+
+    QFile file( path );
+    if ( !file.open( QIODevice::ReadOnly ) )
+        return;
+
+    QDataStream stream( &file );
+    stream.setVersion( 5 );
+
+    qint32 version;
+    stream >> version;
+
+    if ( version != 1 )
+        return;
+
+    qint32 folderId;
+    while ( !( stream >> folderId ).atEnd() && folderId != 0 ) {
+        qint32 state;
+        stream >> state;
+
+        FolderState* fstate = folderState( folderId );
+        fstate->setWatchState( (WatchState)state );
+
+        qint32 issueId;
+        while ( !( stream >> issueId ).atEnd() && issueId != 0 ) {
+            qint32 stamp;
+            stream >> stamp;
+
+            IssueState* istate = issueState( issueId );
+            istate->setWatchStamp( stamp );
+        }
+
+        updateFolderCache( folderId );
+    }
+}
+
+void DataManager::saveWatchCache()
+{
+    QString path = connectionManager->locateCacheFile( "watch.cache" );
+
+    QFile file( path );
+    if ( !file.open( QIODevice::WriteOnly ) )
+        return;
+
+    QDataStream stream( &file );
+    stream.setVersion( 5 );
+
+    stream << (qint32)1; // increment version when adding/changing fields
+
+    RDB::TableConstIterator<FolderRow> itf( &m_folders );
+    while ( itf.next() ) {
+        int folderId = itf.get()->folderId();
+        FolderState* fstate = folderState( folderId );
+
+        if ( fstate->watchState() != NotWatched ) {
+            stream << (qint32)folderId;
+            stream << (qint32)fstate->watchState();
+
+            RDB::ForeignConstIterator<IssueRow> iti( m_issues.parentIndex(), folderId );
+            while ( iti.next() ) {
+                int issueId = iti.get()->issueId();
+                IssueState* istate = issueState( issueId );
+
+                if ( istate->watchStamp() > 0 ) {
+                    stream << (qint32)issueId;
+                    stream << (qint32)istate->watchStamp();
+                }
+            }
+            stream << (qint32)0;
+        }
+    }
+    stream << (qint32)0;
+}
