@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@ using namespace WebIssues;
 
 FolderState::FolderState( int folderId ) :
     m_listStamp( 0 ),
-    m_cached( false )
+    m_cached( false ),
+    m_watchState( NotWatched )
 {
     setKey( 0, folderId );
 }
@@ -26,7 +27,8 @@ FolderState::~FolderState()
 
 IssueState::IssueState( int issueId ) :
     m_detailsStamp( 0 ),
-    m_lockCount( 0 )
+    m_lockCount( 0 ),
+    m_watchStamp( 0 )
 {
     setKey( 0, issueId );
 }
