@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 class QCheckBox;
 class QComboBox;
 class QSpinBox;
+class QLineEdit;
+class QNetworkProxy;
 
 namespace WebIssues
 {
@@ -44,15 +46,25 @@ public: // overrides
     void accept();
 
 private slots:
-    void apply();
+    bool apply();
 
     void dockChanged();
+    void proxyChanged();
+
+private:
+    static bool compareProxies( const QNetworkProxy& lhs, const QNetworkProxy& rhs );
 
 private:
     QCheckBox* m_dockCheckBox;
     QComboBox* m_showComboBox;
     QComboBox* m_reconnectComboBox;
 
+    QComboBox* m_proxyCombo;
+    QLineEdit* m_proxyHostEdit;
+    QLineEdit* m_proxyPortEdit;
+    QLineEdit* m_proxyUserEdit;
+    QLineEdit* m_proxyPasswordEdit;
+
     QSpinBox* m_intervalSpin;
     QComboBox* m_actionComboBox;
     QSpinBox* m_cacheSpin;
