@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
 
 using namespace WebIssues;
 
+/* TRANSLATOR WebIssues::AgeValueEditor */
+
 AgeValueEditor::AgeValueEditor( QObject* parent, QWidget* parentWidget ) :
     AbstractValueEditor( parent )
 {
@@ -40,5 +42,3 @@ QString AgeValueEditor::value()
     QSpinBox* spin = (QSpinBox*)widget();
     return QString::number( spin->value() * 86400 );
 }
-
-#include "filtervalueeditors.moc"
