@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -15,11 +15,8 @@
 #include <QHeaderView>
 #include <QLayout>
 #include <QLabel>
-#include <QComboBox>
 #include <QAction>
 #include <QMenu>
-#include <QMessageBox>
-#include <QInputDialog>
 
 #include "models/tableitemmodel.h"
 #include "models/tablemodels.h"
@@ -27,14 +24,18 @@
 #include "models/columnconditionsettings.h"
 #include "models/treeviewsettings.h"
 #include "models/treeviewhelper.h"
+#include "models/tablemodelshelper.h"
 #include "data/datamanager.h"
 #include "data/updateevent.h"
 #include "data/updatebatch.h"
 #include "data/connectioninfo.h"
 #include "widgets/searcheditbox.h"
+#include "widgets/separatorcombobox.h"
 #include "dialogs/issuedialogs.h"
 #include "dialogs/filterdialog.h"
 #include "dialogs/columnsettingsdialog.h"
+#include "dialogs/managefiltersdialog.h"
+#include "dialogs/savefilterdialog.h"
 #include "xmlui/builder.h"
 #include "connectionmanager.h"
 #include "viewmanager.h"
@@ -42,10 +43,13 @@
 
 using namespace WebIssues;
 
+/* TRANSLATOR WebIssues::FolderView */
+
 FolderView::FolderView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     m_model( NULL ),
     m_filter( NULL ),
-    m_filterType( NoFilter )
+    m_filterType( NoFilter ),
+    m_searchColumn( Column_Name )
 {
     QAction* action;
 
@@ -73,13 +77,9 @@ FolderView::FolderView( QObject* parent, QWidget* parentWidget ) : View( parent
     connect( action, SIGNAL( triggered() ), this, SLOT( saveFilterAs() ) );
     setAction( "saveFilterAs", action );
 
-    action = new QAction( tr( "&Rename Filter..." ), this );
-    connect( action, SIGNAL( triggered() ), this, SLOT( renameFilter() ) );
-    setAction( "renameFilter", action );
-
-    action = new QAction( tr( "&Delete Filter" ), this );
-    connect( action, SIGNAL( triggered() ), this, SLOT( deleteFilter() ) );
-    setAction( "deleteFilter", action );
+    action = new QAction( IconLoader::icon( "folder-filter" ), tr( "&Manage Filters..." ), this );
+    connect( action, SIGNAL( triggered() ), this, SLOT( manageFilters() ) );
+    setAction( "manageFilters", action );
 
     action = new QAction( IconLoader::icon( "configure-columns" ), tr( "C&onfigure Columns..." ), this );
     connect( action, SIGNAL( triggered() ), this, SLOT( configureColumns() ) );
@@ -105,7 +105,7 @@ FolderView::FolderView( QObject* parent, QWidget* parentWidget ) : View( parent
     QLabel* filterLabel = new QLabel( tr( "Filte&r:" ), main );
     filterLayout->addWidget( filterLabel );
 
-    m_filterCombo = new QComboBox( main );
+    m_filterCombo = new SeparatorComboBox( main );
     m_filterCombo->setMinimumWidth( 200 );
 
     connect( m_filterCombo, SIGNAL( activated( int ) ), this, SLOT( filterActivated( int ) ) );
@@ -116,12 +116,25 @@ FolderView::FolderView( QObject* parent, QWidget* parentWidget ) : View( parent
 
     filterLayout->addStretch( 1 );
 
+    QLabel* searchLabel = new QLabel( tr( "&Quick Search:" ), main );
+    filterLayout->addWidget( searchLabel );
+
     m_searchBox = new SearchEditBox( main );
+    m_searchBox->setMinimumWidth( 200 );
 
     connect( m_searchBox, SIGNAL( textChanged( const QString& ) ), this, SLOT( quickSearchChanged( const QString& ) ) );
 
     filterLayout->addWidget( m_searchBox );
 
+    searchLabel->setBuddy( m_searchBox );
+
+    m_searchMenu = new QMenu( m_searchBox );
+    m_searchBox->setOptionsMenu( m_searchMenu );
+
+    m_searchActionGroup = new QActionGroup( this );
+
+    connect( m_searchActionGroup, SIGNAL( triggered( QAction* ) ), this, SLOT( searchActionTriggered( QAction* ) ) );
+
     m_list = new QTreeView( main );
     m_list->setSortingEnabled( true );
     m_list->setRootIsDecorated( false );
@@ -129,8 +142,8 @@ FolderView::FolderView( QObject* parent, QWidget* parentWidget ) : View( parent
 
     connect( m_list, SIGNAL( customContextMenuRequested( const QPoint& ) ),
         this, SLOT( listContextMenu( const QPoint& ) ) );
-    connect( m_list, SIGNAL( doubleClicked( const QModelIndex& ) ),
-        this, SLOT( doubleClicked( const QModelIndex& ) ) );
+    connect( m_list, SIGNAL( activated( const QModelIndex& ) ),
+        this, SLOT( activated( const QModelIndex& ) ) );
 
     mainLayout->addWidget( m_list );
 
@@ -205,17 +218,25 @@ void FolderView::enableView()
     TreeViewSettings settings;
     settings.openIssuesList( m_typeId );
 
-    m_model->setColumns( settings.loadColumns() );
+    QList<Column> columns = settings.loadColumns();
+
+    m_model->setColumns( columns );
 
     m_list->setModel( m_model );
 
-    m_list->sortByColumn( 0, Qt::AscendingOrder );
+    int index = columns.indexOf( Column_ID );
+    if ( index >= 0 )
+        m_list->sortByColumn( index, Qt::AscendingOrder );
+    else
+        m_list->sortByColumn( 0, Qt::AscendingOrder );
 
     TreeViewHelper::applyColumnWidths( m_list, settings.loadColumnWidths() );
 
     connect( m_list->selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
         this, SLOT( updateActions() ) );
 
+    updateSearchOptions();
+
     updateCaption();
     updateActions();
 }
@@ -248,8 +269,6 @@ void FolderView::updateActions()
 
     action( "openIssue" )->setEnabled( m_selectedIssueId != 0 );
     action( "saveFilterAs" )->setEnabled( m_filterType == NewFilter );
-    action( "renameFilter" )->setEnabled( m_filterType == SettingsFilter );
-    action( "deleteFilter" )->setEnabled( m_filterType == SettingsFilter );
 }
 
 void FolderView::updateSummary()
@@ -293,6 +312,8 @@ void FolderView::changeFilter()
     if ( isEnabled() ) {
         FilterDialog dialog( m_projectId, mainWidget() );
 
+        dialog.setPrompt( tr( "Change filter settings for the current view:" ) );
+
         TreeViewSettings settings;
         settings.openIssuesList( m_typeId );
 
@@ -308,75 +329,30 @@ void FolderView::changeFilter()
 void FolderView::saveFilterAs()
 {
     if ( isEnabled() && m_filterType == NewFilter ) {
-        QString name = QInputDialog::getText( mainWidget(), tr( "Save Filter As" ),
-            tr( "Enter the name of the filter to save:" ), QLineEdit::Normal, tr( "New Filter" ) );
-
-        if ( !name.isNull() ) {
-            ColumnConditionSettings settings;
-            settings.openIssueFilters( m_typeId );
-
-            if ( settings.filterExists( name ) ) {
-                if ( QMessageBox::warning( mainWidget(), tr( "Warning" ),
-                    tr( "<qt>Do you want to overwrite the existing filter <b>%1</b>?</qt>" ).arg( name ),
-                    QMessageBox::Yes | QMessageBox::No ) == QMessageBox::No )
-                    return;
-            }
-
-            settings.saveFilter( name, m_filter->conditions() );
-
-            m_filterType = SettingsFilter;
-            m_filterName = name;
-
-            m_newConditions.clear();
-
-            viewManager->postViewEvent( metaObject()->className(), ViewEvent::UpdateFilters, m_typeId );
-        }
-    }
-}
-
-void FolderView::renameFilter()
-{
-    if ( isEnabled() && m_filterType == SettingsFilter ) {
-        QString name = QInputDialog::getText( mainWidget(), tr( "Rename Filter" ),
-            tr( "Enter the new name of the filter:" ), QLineEdit::Normal, m_filterName );
-
-        if ( !name.isNull() ) {
-            ColumnConditionSettings settings;
-            settings.openIssueFilters( m_typeId );
-
-            if ( settings.filterExists( name ) ) {
-                if ( QMessageBox::warning( mainWidget(), tr( "Warning" ),
-                    tr( "<qt>Do you want to overwrite the existing filter <b>%1</b>?</qt>" ).arg( name ),
-                    QMessageBox::Yes | QMessageBox::No ) == QMessageBox::No )
-                    return;
-            }
-
-            settings.renameFilter( m_filterName, name );
-
-            m_filterType = SettingsFilter;
-            m_filterName = name;
-
-            viewManager->postViewEvent( metaObject()->className(), ViewEvent::UpdateFilters, m_typeId );
-        }
-    }
-}
-
-void FolderView::deleteFilter()
-{
-    if ( isEnabled() && m_filterType == SettingsFilter ) {
-        if ( QMessageBox::warning( mainWidget(), tr( "Delete Filter" ),
-            tr( "<qt>Are you sure you want to delete filter <b>%1</b>?</qt>" ).arg( m_filterName ),
-            QMessageBox::Yes | QMessageBox::No ) == QMessageBox::No )
-            return;
-
         ColumnConditionSettings settings;
         settings.openIssueFilters( m_typeId );
 
-        settings.deleteFilter( m_filterName );
+        SaveFilterDialog dialog( mainWidget() );
+        dialog.setExistingFilters( settings.filterNames() );
 
-        m_filterType = NoFilter;
+        if ( dialog.exec() == QDialog::Accepted ) {
+            settings.saveFilter( dialog.filterName(), m_filter->conditions() );
 
-        viewManager->postViewEvent( metaObject()->className(), ViewEvent::UpdateFilters, m_typeId );
+            m_filterType = SettingsFilter;
+            m_filterName = dialog.filterName();
+
+            m_newConditions.clear();
+
+            viewManager->postViewEvent( NULL, ViewEvent::UpdateFilters, m_typeId );
+        }
+    }
+}
+
+void FolderView::manageFilters()
+{
+    if ( isEnabled() ) {
+        ManageFiltersDialog dialog( id(), mainWidget() );
+        dialog.exec();
     }
 }
 
@@ -395,7 +371,7 @@ void FolderView::configureColumns()
 
         dialog.setAvailableColumns( settings.availableColumns() );
         dialog.setDefaultColumns( settings.defaultColumns() );
-        dialog.setFixedColumns( 2 );
+        dialog.setFixedColumns( settings.fixedColumns() );
 
         dialog.setColumns( settings.loadColumns() );
 
@@ -438,12 +414,31 @@ void FolderView::updateEvent( UpdateEvent* e )
 
 void FolderView::viewEvent( ViewEvent* e )
 {
-    if ( isEnabled() && e->action() == ViewEvent::UpdateColumnList && e->id() == m_typeId )
-        updateColumnList();
-    else if ( isEnabled() && e->action() == ViewEvent::UpdateFilters && e->id() == m_typeId )
-        updateFilters();
-    else
-        View::viewEvent( e );
+    if ( isEnabled() ) {
+        switch ( e->action() ) {
+            case ViewEvent::UpdateColumnList:
+                if ( e->id() == m_typeId )
+                    updateColumnList();
+                break;
+
+            case ViewEvent::UpdateFilters:
+                if ( e->id() == m_typeId )
+                    updateFilters();
+                break;
+
+            case ViewEvent::UpdateSettings:
+                updateColumnList();
+                updateFilters();
+                break;
+
+            case ViewEvent::RecalculateWatches:
+            case ViewEvent::UpdateWatches:
+                if ( e->id() == id() )
+                    updateColumnList();
+                break;
+        }
+    }
+    View::viewEvent( e );
 }
 
 void FolderView::headerContextMenu( const QPoint& pos )
@@ -468,7 +463,7 @@ void FolderView::listContextMenu( const QPoint& pos )
         menu->exec( m_list->viewport()->mapToGlobal( pos ) );
 }
 
-void FolderView::doubleClicked( const QModelIndex& index )
+void FolderView::activated( const QModelIndex& index )
 {
     if ( index.isValid() ) {
         int issueId = m_model->data( index, TableItemModel::RowIdRole ).toInt();
@@ -488,7 +483,7 @@ void FolderView::updateColumnList()
 
     QList<Column> columns = settings.loadColumns();
 
-    m_model->setColumns( settings.loadColumns() );
+    m_model->setColumns( columns );
 
     int index = columns.indexOf( sortColumn );
     if ( index >= 0 )
@@ -497,11 +492,58 @@ void FolderView::updateColumnList()
         m_list->sortByColumn( 0, Qt::AscendingOrder );
 
     TreeViewHelper::applyColumnWidths( m_list, settings.loadColumnWidths() );
+
+    updateSearchOptions();
+}
+
+void FolderView::updateSearchOptions()
+{
+    if ( !m_model->columns().contains( m_searchColumn ) ) {
+        m_searchColumn = Column_Name;
+        m_filter->setQuickSearch( m_searchColumn, m_searchBox->text() );
+    }
+
+    m_searchMenu->clear();
+
+    for ( int i = 0; i < m_model->columns().count(); i++ )
+    {
+        Column column = m_model->columns().at( i );
+        QString name = TableModelsHelper::columnName( column );
+        QAction* action = m_searchMenu->addAction( name );
+        action->setData( (int)column );
+        action->setCheckable( true );
+        if ( column == m_searchColumn )
+            action->setChecked( true );
+        action->setActionGroup( m_searchActionGroup );
+    }
+
+    QString prompt = TableModelsHelper::columnName( m_searchColumn );
+    m_searchBox->setPromptText( prompt );
 }
 
 void FolderView::quickSearchChanged( const QString& text )
 {
-    m_filter->setQuickSearch( text );
+    m_filter->setQuickSearch( m_searchColumn, text );
+}
+
+void FolderView::searchActionTriggered( QAction* action )
+{
+    m_searchColumn = (Column)action->data().toInt();
+
+    QString prompt = TableModelsHelper::columnName( m_searchColumn );
+    m_searchBox->setPromptText( prompt );
+
+    m_filter->setQuickSearch( m_searchColumn, m_searchBox->text() );
+}
+
+void FolderView::selectFilter( const QString& filter )
+{
+    m_filterType = SettingsFilter;
+    m_filterName = filter;
+
+    m_searchBox->clear();
+
+    updateFilters();
 }
 
 void FolderView::updateFilters()
@@ -518,12 +560,13 @@ void FolderView::updateFilters()
     if ( filters.count() > 0 ) {
         filters.sort();
 
+        m_filterCombo->addSeparator();
         m_filterCombo->addItems( filters );
 
         if ( m_filterType == SettingsFilter ) {
             for ( int i = 0; i < filters.count(); i++ ) {
                 if ( filters[ i ] == m_filterName ) {
-                    m_filterCombo->setCurrentIndex( i + 1 );
+                    m_filterCombo->setCurrentIndex( i + 2 );
                     break;
                 }
             }
@@ -531,6 +574,7 @@ void FolderView::updateFilters()
     }
 
     if (  m_newConditions.count() > 0 ) {
+        m_filterCombo->addSeparator();
         m_filterCombo->addItem( tr( "(Unnamed Filter)" ) );
         if ( m_filterType == NewFilter )
             m_filterCombo->setCurrentIndex( m_filterCombo->count() - 1 );
@@ -605,5 +649,3 @@ QModelIndex FolderView::selectedIndex()
 
     return selection.at( 0 );
 }
-
-#include "folderview.moc"
