@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007 WebIssues Team
+* Copyright (C) 2007-2008 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@
 #include "widgets/popeditbox.h"
 #include "dialogs/issuedialogs.h"
 #include "dialogs/finditemdialog.h"
-#include "dialogs/finddialog.h"
+#include "dialogs/findbar.h"
 #include "dialogs/checkmessagebox.h"
 #include "dialogs/attributesettingsdialog.h"
 #include "xmlui/builder.h"
@@ -62,9 +62,10 @@
 
 using namespace WebIssues;
 
+/* TRANSLATOR WebIssues::IssueView */
+
 IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     m_model( NULL ),
-    m_findDialog( NULL ),
     m_gotoItemId( 0 )
 {
     QAction* action;
@@ -74,7 +75,7 @@ IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     connect( action, SIGNAL( triggered() ), this, SLOT( updateIssue() ) );
     setAction( "updateIssue", action );
 
-    action = new QAction( IconLoader::icon( "comment" ), tr( "Add &Comment..." ), this );
+    action = new QAction( IconLoader::icon( "comment" ), tr( "Add C&omment..." ), this );
     action->setShortcut( QKeySequence::New );
     connect( action, SIGNAL( triggered() ), this, SLOT( addComment() ) );
     setAction( "addComment", action );
@@ -155,6 +156,8 @@ IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     connect( m_tabWidget, SIGNAL( currentChanged( int ) ), this, SLOT( tabChanged( int ) ) );
 
     QWidget* issueTab = new QWidget( m_tabWidget );
+    issueTab->setAutoFillBackground( true );
+
     QVBoxLayout* issueLayout = new QVBoxLayout( issueTab );
     QHBoxLayout* nameLayout = new QHBoxLayout();
     issueLayout->addLayout( nameLayout );
@@ -206,10 +209,16 @@ IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     QWidget* historyTab = new QWidget( m_tabWidget );
     QVBoxLayout* historyLayout = new QVBoxLayout( historyTab );
     historyLayout->setMargin( 0 );
+    historyLayout->setSpacing( 0 );
 
     m_browser = new QTextBrowser( historyTab );
     m_browser->setContextMenuPolicy( Qt::CustomContextMenu );
 
+    QPalette palette = m_browser->palette();
+    palette.setBrush( QPalette::Inactive, QPalette::Highlight, palette.brush( QPalette::Active, QPalette::Highlight ) );
+    palette.setBrush( QPalette::Inactive, QPalette::HighlightedText, palette.brush( QPalette::Active, QPalette::HighlightedText ) );
+    m_browser->setPalette( palette );
+
     historyLayout->addWidget( m_browser );
 
     connect( m_browser, SIGNAL( customContextMenuRequested( const QPoint& ) ),
@@ -217,6 +226,18 @@ IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
     connect( m_browser, SIGNAL( anchorClicked( const QUrl& ) ), this, SLOT( anchorClicked( const QUrl& ) ) );
     connect( m_browser, SIGNAL( selectionChanged() ), this, SLOT( updateActions() ) );
 
+    m_findBar = new FindBar( historyTab );
+    m_findBar->setBoundWidget( m_browser );
+    m_findBar->setAutoFillBackground( true );
+    m_findBar->hide();
+
+    historyLayout->addWidget( m_findBar );
+
+    connect( m_findBar, SIGNAL( find( const QString& ) ), this, SLOT( findText( const QString& ) ) );
+    connect( m_findBar, SIGNAL( findPrevious() ), this, SLOT( findPrevious() ) );
+    connect( m_findBar, SIGNAL( findNext() ), this, SLOT( findNext() ) );
+    connect( m_findBar, SIGNAL( findEnabled( bool ) ), this, SLOT( updateActions() ) );
+
     QWidget* attachmentsTab = new QWidget( m_tabWidget );
     QVBoxLayout* attachmentsLayout = new QVBoxLayout( attachmentsTab );
     attachmentsLayout->setMargin( 0 );
@@ -230,8 +251,8 @@ IssueView::IssueView( QObject* parent, QWidget* parentWidget ) : View( parent ),
 
     connect( m_list, SIGNAL( customContextMenuRequested( const QPoint& ) ),
         this, SLOT( attachmentsContextMenu( const QPoint& ) ) );
-    connect( m_list, SIGNAL( doubleClicked( const QModelIndex& ) ),
-        this, SLOT( doubleClicked( const QModelIndex& ) ) );
+    connect( m_list, SIGNAL( activated( const QModelIndex& ) ),
+        this, SLOT( activated( const QModelIndex& ) ) );
 
     m_tabWidget->addTab( issueTab, IconLoader::icon( "issue-open" ), tr( "Issue Details" ) );
     m_tabWidget->addTab( historyTab, IconLoader::icon( "view-history" ), tr( "Issue History" ) );
@@ -268,8 +289,12 @@ void IssueView::initialUpdate()
 
     dataManager->lockIssue( id() );
 
-    if ( dataManager->issueUpdateNeeded( id() ) )
+    if ( dataManager->issueUpdateNeeded( id() ) ) {
         initialUpdateIssue();
+    } else {
+        dataManager->updateIssueWatchStamp( id() );
+        viewManager->postViewEvent( NULL, ViewEvent::RecalculateWatches, m_folderId );
+    }
 }
 
 Access IssueView::checkDataAccess()
@@ -331,12 +356,10 @@ void IssueView::disableView()
     m_attributePanel->clear();
 
     m_browser->clear();
+    m_findBar->hide();
 
     m_list->setModel( NULL );
 
-    delete m_findDialog;
-    m_findDialog = NULL;
-
     updateCaption();
 }
 
@@ -390,7 +413,7 @@ void IssueView::updateActions()
 
     m_currentPage = m_tabWidget->currentIndex();
 
-    m_isFindEnabled = m_findDialog && m_findDialog->isFindEnabled();
+    m_isFindEnabled = m_findBar->isFindEnabled();
 
     bool hasSelection = m_browser->textCursor().hasSelection();
 
@@ -505,54 +528,66 @@ void IssueView::selectAll()
 void IssueView::find()
 {
     if ( isEnabled() && m_currentPage == 1 ) {
-        if ( !m_findDialog ) {
-            m_findDialog = new FindDialog( mainWidget() );
-
-            connect( m_findDialog, SIGNAL( findClicked() ), this, SLOT( findClicked() ) );
-            connect( m_findDialog, SIGNAL( findEnabled( bool ) ), this, SLOT( updateActions() ) );
-        }
-
-        m_findDialog->show();
-        m_findDialog->activateWindow();
+        m_findBar->show();
+        m_findBar->setFocus();
     }
 }
 
-void IssueView::findClicked()
+void IssueView::findText( const QString& text )
 {
-    m_tabWidget->setCurrentIndex( 1 );
-
-    findNext();
-
-    viewManager->activateView( this );
-    m_browser->setFocus();
+    if ( isEnabled() && m_currentPage == 1 )
+        findText( text, m_browser->textCursor().selectionStart(), m_findBar->flags() );
 }
 
 void IssueView::findNext()
 {
-    if ( isEnabled() && m_currentPage == 1 && m_isFindEnabled ) {
-        if ( !m_browser->find( m_findDialog->text(), m_findDialog->flags() ) ) {
-            m_browser->moveCursor( QTextCursor::Start );
-
-            if ( !m_browser->find( m_findDialog->text(), m_findDialog->flags() ) )
-                showInfo( tr( "Text not found" ) );
-        }
-    }
+    if ( isEnabled() && m_currentPage == 1 && m_isFindEnabled )
+        findText( m_findBar->text(), m_browser->textCursor().selectionStart() + 1, m_findBar->flags() );
 }
 
 void IssueView::findPrevious()
 {
-    if ( isEnabled() && m_currentPage == 1 && m_isFindEnabled ) {
-        QTextCursor cursor = m_browser->textCursor();
-        cursor.setPosition( cursor.selectionStart() );
-        m_browser->setTextCursor( cursor );
+    if ( isEnabled() && m_currentPage == 1 && m_isFindEnabled )
+        findText( m_findBar->text(), m_browser->textCursor().selectionStart(), m_findBar->flags() | QTextDocument::FindBackward );
+}
 
-        if ( !m_browser->find( m_findDialog->text(), m_findDialog->flags() | QTextDocument::FindBackward ) ) {
-            m_browser->moveCursor( QTextCursor::End );
+void IssueView::findText( const QString& text, int from, QTextDocument::FindFlags flags )
+{
+    QTextCursor found;
+    bool warn = false;
 
-            if ( !m_browser->find( m_findDialog->text(), m_findDialog->flags() | QTextDocument::FindBackward ) )
-                showInfo( tr( "Text not found" ) );
+    if ( !text.isEmpty() ) {
+        found = m_browser->document()->find( text, from, flags );
+
+        if ( found.isNull() ) {
+            if ( flags & QTextDocument::FindBackward ) {
+                QTextCursor end( m_browser->document() );
+                end.movePosition( QTextCursor::End );
+                from = end.position();
+            } else {
+                from = 0;
+            }
+
+            found = m_browser->document()->find( text, from, flags );
+
+            if ( found.isNull() )
+                warn = true;
         }
     }
+
+    if ( found.isNull() ) {
+        found = m_browser->textCursor();
+        found.setPosition( found.selectionStart() );
+    }
+
+    m_browser->setTextCursor( found );
+
+    m_findBar->showWarning( warn );
+
+    if ( warn ) {
+        m_findBar->show();
+        m_findBar->setFocus();
+    }
 }
 
 void IssueView::openAttachment()
@@ -599,6 +634,9 @@ void IssueView::updateEvent( UpdateEvent* e )
             updateProperties();
             updateAttributes();
             populateComments();
+
+            dataManager->updateIssueWatchStamp( id() );
+            viewManager->postViewEvent( NULL, ViewEvent::RecalculateWatches, m_folderId );
         }
 
         if ( e->unit() == UpdateEvent::Users ) {
@@ -635,10 +673,19 @@ void IssueView::updateEvent( UpdateEvent* e )
 
 void IssueView::viewEvent( ViewEvent* e )
 {
-    if ( isEnabled() && e->action() == ViewEvent::UpdateAttributes && e->id() == m_typeId )
-        populateAttributes();
-    else
-        View::viewEvent( e );
+    if ( isEnabled() ) {
+        switch ( e->action() ) {
+            case ViewEvent::UpdateAttributes:
+                if ( e->id() == m_typeId )
+                    populateAttributes();
+                break;
+
+            case ViewEvent::UpdateSettings:
+                populateAttributes();
+                break;
+        }
+    }
+    View::viewEvent( e );
 }
 
 void IssueView::populateProperties()
@@ -819,7 +866,7 @@ void IssueView::attachmentsContextMenu( const QPoint& pos )
         menu->exec( m_list->viewport()->mapToGlobal( pos ) );
 }
 
-void IssueView::doubleClicked( const QModelIndex& index )
+void IssueView::activated( const QModelIndex& index )
 {
     if ( index.isValid() ) {
         int attachmenId = m_model->data( index, TableItemModel::RowIdRole ).toInt();
@@ -964,5 +1011,3 @@ QModelIndex IssueView::selectedIndex()
 
     return selection.at( 0 );
 }
-
-#include "issueview.moc"
